﻿Imports System.Math
Imports System.Drawing.Drawing2D

Public Class Form1
    ' The circles.
    Private Centers As New List(Of PointF)()
    Private Radii As New List(Of Single)()

    ' The index of a new circle we are drawing.
    Private NewCircle As Integer = -1

    ' Remove all circles.
    Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClear.Click
        Centers = New List(Of PointF)()
        Radii = New List(Of Single)()
        Me.Invalidate()
    End Sub

    ' Start drawing a new circle.
    Private Sub Form1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        Centers.Add(e.Location)
        Radii.Add(0)
        NewCircle = Centers.Count - 1
    End Sub

    ' Update the new circle.
    Private Sub Form1_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If (NewCircle < 0) Then Return
        Dim dx As Single = e.X - Centers(NewCircle).X
        Dim dy As Single = e.Y - Centers(NewCircle).Y
        Radii(NewCircle) = CSng(Sqrt(dx * dx + dy * dy))
        Me.Invalidate()
    End Sub

    ' Finish drawing a new circle.
    Private Sub Form1_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        ' If the radius is 0, remove the new circle.
        If (Radii(NewCircle) < 1) Then
            Centers.RemoveAt(NewCircle)
            Radii.RemoveAt(NewCircle)
        End If

        NewCircle = -1
        Me.Invalidate()
    End Sub

    ' Draw the circles.
    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias

        ' Find the intersection of all circles.
        Dim intersection As Region = FindCircleIntersections(Centers, Radii)

        ' Draw the region.
        If (intersection IsNot Nothing) Then
            e.Graphics.FillRegion(Brushes.LightGreen, intersection)

            ' Get the region's centroid.
            Dim centroid As PointF = RegionCentroid(intersection, e.Graphics.Transform)
            e.Graphics.FillEllipse(Brushes.Red, _
                centroid.X - 3, centroid.Y - 3, 7, 7)
        End If

        ' Draw the circles.
        For i As Integer = 0 To Centers.Count - 1
            e.Graphics.DrawEllipse(Pens.Blue, _
                Centers(i).X - Radii(i), Centers(i).Y - Radii(i), _
                2 * Radii(i), 2 * Radii(i))
        Next i
    End Sub

    ' Find the intersection of all of the circles.
    Private Function FindCircleIntersections(ByVal centers As List(Of PointF), ByVal radii As List(Of Single)) As Region
        If (centers.Count < 1) Then Return Nothing

        ' Make a region.
        Dim result_region As New Region()

        ' Intersect the region with the circles.
        For i As Integer = 0 To centers.Count - 1
            Using circle_path As New GraphicsPath()
                circle_path.AddEllipse( _
                    centers(i).X - radii(i), centers(i).Y - radii(i), _
                    2 * radii(i), 2 * radii(i))
                result_region.Intersect(circle_path)
            End Using
        Next i

        Return result_region
    End Function

    ' Return the centroid of the region.
    Private Function RegionCentroid(ByVal the_region As Region, ByVal transform As Matrix) As PointF
        Dim cx As Single = 0
        Dim cy As Single = 0
        Dim total_weight As Single = 0
        For Each rect As RectangleF In the_region.GetRegionScans(transform)
            Dim rect_weight As Single = rect.Width * rect.Height
            cx += rect_weight * (rect.Left + rect.Width / 2.0F)
            cy += rect_weight * (rect.Top + rect.Height / 2.0F)
            total_weight += rect_weight
        Next rect

        Return New PointF(cx / total_weight, cy / total_weight)
    End Function
End Class
